@@ -678,9 +678,33 @@ class TransformShape(ShapeElement, Transform):
         self.anchor_point = MultiDimensional(NVector(0, 0))
 
 
+## \ingroup Lottie
+class Composite(TgsEnum):
+    Above = 1
+    Below = 2
+
+
+## \ingroup Lottie
+class RepeaterTransform(Transform):
+    _props = [
+        TgsProp("start_opacity", "so", Value, False),
+        TgsProp("end_opacity", "eo", Value, False),
+    ]
+
+    def __init__(self):
+        Transform.__init__(self)
+        self.start_opacity = Value(100)
+        self.end_opacity = Value(100)
+
+
+## \ingroup Lottie
+class Modifier(ShapeElement):
+    pass
+
+
 ## \ingroup Lottie
 ## \todo Implement SVG/SIF Export
-class Trim(ShapeElement):
+class Trim(Modifier):
     """
     Trims shapes into a segment
     """
@@ -705,30 +729,6 @@ class Trim(ShapeElement):
         self.m = None
 
 
-## \ingroup Lottie
-class Composite(TgsEnum):
-    Above = 1
-    Below = 2
-
-
-## \ingroup Lottie
-class RepeaterTransform(Transform):
-    _props = [
-        TgsProp("start_opacity", "so", Value, False),
-        TgsProp("end_opacity", "eo", Value, False),
-    ]
-
-    def __init__(self):
-        Transform.__init__(self)
-        self.start_opacity = Value(100)
-        self.end_opacity = Value(100)
-
-
-## \ingroup Lottie
-class Modifier(ShapeElement):
-    pass
-
-
 ## \ingroup Lottie
 class Repeater(Modifier):
     """
