@@ -13,6 +13,19 @@ except ImportError:
     has_font = False
 
 
+class PrecompTime:
+    def __init__(self, pcl: objects.PreCompLayer):
+        self.pcl = pcl
+
+    def get_time_offset(self, time, lot):
+        remap = time
+        if self.pcl.time_remapping:
+            remapf = self.pcl.time_remapping.get_value(time)
+            remap = lot.in_point * (1-remapf) + lot.out_point * remapf
+
+        return remap - self.pcl.start_time
+
+
 class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
     merge_paths = True
     namestart = (
@@ -31,7 +44,19 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         self.ids = set()
         self.idc = 0
         self.name_mode = NameMode.Inkscape
-        self.time = time
+        self.actual_time = time
+        self.precomp_times = []
+        self._precomps = {}
+        self._assets = {}
+        self._current_layer = []
+
+    @property
+    def time(self):
+        time = self.actual_time
+        if self.precomp_times:
+            for pct in self.precomp_times:
+                time = pct.get_time_offset(time, self._current_layer[-1])
+        return time
 
     def gen_id(self, prefix="id"):
         #TODO should check if id_n already exists
@@ -40,6 +65,16 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         self.ids.add(id)
         return id
 
+    def set_clean_id(self, dom, n):
+        idn = n.replace(" ", "_")
+        if self.id_re.match(idn) and idn not in self.ids:
+            self.ids.add(idn)
+        else:
+            idn = self.gen_id(dom.tag)
+
+        dom.attrib["id"] = idn
+        return idn
+
     def set_id(self, dom, lottieobj, inkscape_qual=None, force=False):
         n = getattr(lottieobj, "name", None)
         if n is None or self.name_mode == NameMode.NoName:
@@ -49,16 +84,10 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
                 return id
             return None
 
-        idn = n.replace(" ", "_")
-        if self.id_re.match(idn) and idn not in self.ids:
-            self.ids.add(idn)
-        else:
-            idn = self.gen_id(dom.tag)
-
-        dom.attrib["id"] = idn
+        idn = self.set_clean_id(dom, n)
         if inkscape_qual:
             dom.attrib[inkscape_qual] = n
-        return n
+        return idn
 
     def _on_animation(self, animation: objects.Animation):
         self.svg.attrib["width"] = str(animation.width)
@@ -77,20 +106,64 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
             namedview.attrib["pagecolor"] = "#ffffff"
         self.svg.attrib["style"] = "fill: none; stroke: none"
 
+        self._current_layer = [animation]
         return self.svg
 
+    def _mask_to_def(self, mask):
+        svgmask = ElementTree.SubElement(self.defs, "mask")
+        mask_id = self.gen_id()
+        svgmask.attrib["id"] = mask_id
+        svgmask.attrib["mask-type"] = "alpha"
+        path = ElementTree.SubElement(svgmask, "path")
+        path.attrib["d"] = self._bezier_to_d(mask.shape.get_value(self.time))
+        path.attrib["fill"] = "#fff"
+        path.attrib["fill-opacity"] = str(mask.opacity.get_value(self.time) / 100)
+        return mask_id
+
+    def _on_masks(self, masks):
+        if len(masks) == 1:
+            return self._mask_to_def(masks[0])
+        mask_ids = list(map(self._mask_to_def, masks))
+        mask_def = ElementTree.SubElement(self.defs, "mask")
+        mask_id = self.gen_id()
+        mask_def.attrib["id"] = mask_id
+        g = mask_def
+        for mid in mask_ids:
+            g = ElementTree.SubElement(g, "g")
+            g.attrib["mask"] = "url(#%s)" % mid
+        full = ElementTree.SubElement(g, "rect")
+        full.attrib["fill"] = "#fff"
+        full.attrib["width"] = self.svg.attrib["width"]
+        full.attrib["height"] = self.svg.attrib["height"]
+        full.attrib["x"] = "0"
+        full.attrib["y"] = "0"
+        return mask_id
+
     def _on_layer(self, layer_builder, dom_parent):
         lot = layer_builder.lottie
-        if (lot.in_point > self.time or lot.out_point < self.time) and dom_parent not in self.defs:
+        self._current_layer.append(lot)
+
+        if not self.precomp_times and (lot.in_point > self.time or lot.out_point < self.time):
+            self._current_layer.pop()
             return None
 
         g = self.group_from_lottie(lot, dom_parent, True)
 
+        if lot.masks:
+            g.attrib["mask"] = "url(#%s)" % self._on_masks(lot.masks)
+
         if isinstance(lot, objects.PreCompLayer):
-            use = ElementTree.SubElement(g, "use")
-            use.attrib[self.qualified("xlink", "href")] = "#" + lot.reference_id
+            self.precomp_times.append(PrecompTime(lot))
+
+            for layer in self._precomps.get(lot.reference_id, []):
+                self.process_layer(layer, g)
+
+            self.precomp_times.pop()
         elif isinstance(lot, objects.NullLayer):
             g.attrib["opacity"] = "1"
+        elif isinstance(lot, objects.ImageLayer):
+            use = ElementTree.SubElement(g, "use")
+            use.attrib[self.qualified("xlink", "href")] = "#" + self._assets[lot.image_id]
 
         if not lot.name:
             g.attrib[self.qualified("inkscape", "label")] = lot.__class__.__name__
@@ -102,10 +175,20 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
 
         return g
 
-    def _on_precomp(self, id, dom_parent):
-        g = ElementTree.SubElement(self.defs, "g")
-        g.attrib["id"] = id
-        return g
+    def _on_layer_end(self, out_layer):
+        self._current_layer.pop()
+
+    def _on_precomp(self, id, dom_parent, layers):
+        self._precomps[id] = layers
+
+    def _on_asset(self, asset):
+        if isinstance(asset, objects.assets.Image):
+            img = ElementTree.SubElement(self.defs, "image")
+            xmlid = self.set_clean_id(img, asset.id)
+            self._assets[asset.id] = xmlid
+            img.attrib[self.qualified("xlink", "href")] = asset.image or asset.image_path
+            img.attrib["width"] = str(asset.width)
+            img.attrib["height"] = str(asset.height)
 
     def _get_value(self, prop, default=NVector(0, 0)):
         if prop:
@@ -229,7 +312,7 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
         id = self.set_id(dom, gradient, force=True)
         dom.attrib["gradientUnits"] = "userSpaceOnUse"
 
-        for off, color in gradient.colors.sane_colors_at(self.time):
+        for off, color in gradient.colors.stops_at(self.time):
             stop = ElementTree.SubElement(dom, "stop")
             stop.attrib["offset"] = "%s%%" % (off * 100)
             stop.attrib["stop-color"] = color_to_css(color[:3])
@@ -321,31 +404,36 @@ class SvgBuilder(SvgHandler, restructure.AbstractBuilder):
                 continue
             if d:
                 d += "\n"
-            d += "M %s,%s " % tuple(bez.vertices[0].components[:2])
-            for i in range(1, len(bez.vertices)):
-                qfrom = bez.vertices[i-1]
-                h1 = bez.out_tangents[i-1] + qfrom
-                qto = bez.vertices[i]
-                h2 = bez.in_tangents[i] + qto
-                d += "C %s,%s %s,%s %s,%s " % (
-                    h1[0], h1[1],
-                    h2[0], h2[1],
-                    qto[0], qto[1],
-                )
-            if bez.closed:
-                qfrom = bez.vertices[-1]
-                h1 = bez.out_tangents[-1] + qfrom
-                qto = bez.vertices[0]
-                h2 = bez.in_tangents[0] + qto
-                d += "C %s,%s %s,%s %s,%s Z" % (
-                    h1[0], h1[1],
-                    h2[0], h2[1],
-                    qto[0], qto[1],
-                )
+            d += self._bezier_to_d(bez)
 
         path.attrib["d"] = d
         return path
 
+    def _bezier_to_d(self, bez):
+        d = "M %s,%s " % tuple(bez.vertices[0].components[:2])
+        for i in range(1, len(bez.vertices)):
+            qfrom = bez.vertices[i-1]
+            h1 = bez.out_tangents[i-1] + qfrom
+            qto = bez.vertices[i]
+            h2 = bez.in_tangents[i] + qto
+            d += "C %s,%s %s,%s %s,%s " % (
+                h1[0], h1[1],
+                h2[0], h2[1],
+                qto[0], qto[1],
+            )
+        if bez.closed:
+            qfrom = bez.vertices[-1]
+            h1 = bez.out_tangents[-1] + qfrom
+            qto = bez.vertices[0]
+            h2 = bez.in_tangents[0] + qto
+            d += "C %s,%s %s,%s %s,%s Z" % (
+                h1[0], h1[1],
+                h2[0], h2[1],
+                qto[0], qto[1],
+            )
+
+        return d
+
     def _on_shape_modifier(self, shape, shapegroup, out_parent):
         if isinstance(shape.lottie, objects.Repeater):
             svgshape = self.build_repeater(shape.lottie, shape.child, shapegroup, out_parent)
